import typing

from . import context
from ._post_event import apost_event, post_event

if typing.TYPE_CHECKING:
    import aiohttp
    import requests


def init(api_key: str, api_base_url: str | None = None) -> "requests.Response":
    """
    Initialize the Sticklight client.
    Call this function once at the beginning of your program.
    Captures a 'sticklight_init' event to the Sticklight API.

    Args:
        api_key (str): The API key for the Sticklight client.
        api_base_url (str | None): An optional base URL for the Sticklight API.

    Returns:
        requests.Response: The response from the Sticklight API.
    """
    context.set_api_key(api_key)
    if api_base_url:
        context.set_api_base_url(api_base_url)
    return post_event("sticklight_init", api_base_url=api_base_url)


async def ainit(
    api_key: str, api_base_url: str | None = None
) -> "aiohttp.ClientResponse":
    """
    Initialize the Sticklight client asynchronously.
    """
    context.set_api_key(api_key)
    if api_base_url:
        context.set_api_base_url(api_base_url)
    return await apost_event("sticklight_init", api_base_url=api_base_url)


__all__ = ["ainit", "init"]



from . import context
from .consts import STICKLIGHT_API_KEY_ENV_VAR_NAME
from .errors import SticklightApiKeyNotFoundError


def resolve_sticklight_api_key(sticklight_api_key: str | None = None) -> str:
    """
    Resolve the Sticklight API key with the following order of precedence:
    1. Value passed to the function
    2. Value stored via `sticklight.init(api_key=...)`
    3. Value stored via the `STICKLIGHT_API_KEY` environment variable
    4. If no API key could be resolved, throw a `SticklightApiKeyNotFoundError`
    """
    if sticklight_api_key:
        return sticklight_api_key

    sticklight_api_key = context.get_api_key()
    if not sticklight_api_key:
        error_message = (
            f"Sticklight API key not found. "
            f"Either set the {STICKLIGHT_API_KEY_ENV_VAR_NAME} environment variable or call sticklight.init() with the API key at the start of your program."
        )
        raise SticklightApiKeyNotFoundError(error_message)
    return sticklight_api_key


__all__ = ["SticklightApiKeyNotFoundError", "resolve_sticklight_api_key"]



from . import context as context
from .capture import acapture as acapture, capture as capture
from .init import ainit as ainit, init as init

__all__ = ["acapture", "ainit", "capture", "context", "init"]



import typing

from ._post_event import apost_event, post_event

if typing.TYPE_CHECKING:
    import aiohttp
    import requests


def capture(event_name: str, **event_data) -> "requests.Response":
    """
    Publish an event to Sticklight API.

    Args:
        event_name (str): The name of the event to publish.
        event_data (dict): A free-form dictionary of data to publish.

    Returns:
        requests.Response: The response from the Sticklight API.
    """
    return post_event(event_name, **event_data)


async def acapture(event_name: str, **event_data) -> "aiohttp.ClientResponse":
    """
    Publish an event to Sticklight API asynchronously.

    Args:
        event_name (str): The name of the event to publish.
        event_data (dict): A free-form dictionary of data to publish.

    Returns:
        aiohttp.ClientResponse: The response from the Sticklight API.
    """
    return await apost_event(event_name, **event_data)


__all__ = ["acapture", "capture"]



import typing

from . import auth, context

if typing.TYPE_CHECKING:
    import aiohttp
    import requests


def post_event(event_name: str, **event_data) -> "requests.Response":
    import requests

    api_key = auth.resolve_sticklight_api_key()
    request_body = [{"event_name": event_name, **event_data}]
    response = requests.post(
        f"{context.get_api_base_url()}/events-collect/v1/events",
        json=request_body,
        headers={"accept": "application/json", "x-api-key": api_key},
        timeout=30,
    )
    try:
        response.raise_for_status()
    except Exception as e:
        import logging

        logger = logging.getLogger("sticklight.post_event")
        logger.error("Error posting event %s: %r", event_name, e, stacklevel=2)
    return response


async def apost_event(
    event_name: str, **event_data
) -> "aiohttp.ClientResponse":
    import aiohttp

    api_key = auth.resolve_sticklight_api_key()
    request_body = [{"event_name": event_name, **event_data}]
    async with (
        aiohttp.ClientSession() as session,
        session.post(
            f"{context.get_api_base_url()}/events-collect/v1/events",
            json=request_body,
            headers={"accept": "application/json", "x-api-key": api_key},
            timeout=30,
        ) as response,
    ):
        try:
            response.raise_for_status()
        except Exception as e:
            import logging

            logger = logging.getLogger("sticklight.apost_event")
            logger.error(
                "Error posting event %s: %r", event_name, e, stacklevel=2
            )
        return response


__all__ = ["apost_event", "post_event"]



import os
from contextvars import ContextVar

from sticklight import consts

sticklight_api_key_ctx_var = ContextVar("sticklight_api_key")
sticklight_api_base_url_ctx_var = ContextVar("sticklight_api_base_url")


def get_api_key() -> str:
    return sticklight_api_key_ctx_var.get(None) or os.getenv(
        consts.STICKLIGHT_API_KEY_ENV_VAR_NAME
    )


def set_api_key(api_key: str) -> str:
    if not api_key:
        msg = f"api_key cannot be empty, got: {api_key!r}"
        raise ValueError(msg)
    sticklight_api_key_ctx_var.set(api_key)
    return api_key


def get_api_base_url() -> str:
    return (
        sticklight_api_base_url_ctx_var.get(None)
        or consts.STICKLIGHT_API_BASE_URL
    )


def set_api_base_url(api_base_url: str) -> str:
    if not api_base_url:
        msg = f"api_base_url cannot be empty, got: {api_base_url!r}"
        raise ValueError(msg)
    sticklight_api_base_url_ctx_var.set(api_base_url)
    return api_base_url
